@@ -10,6 +10,7 @@ from sqlglot import expressions as sqlglot_expressions
 from sqlglot.dialects import Dialect
 from sqlglot import tokens
 from .errors import SyncanySqlCompileException
+from .calculaters import is_mysql_func
 from .config import CONST_CONFIG_KEYS
 from .parser import SqlParser
 from .taskers.delete import DeleteTasker
@@ -32,11 +33,17 @@ class CompilerDialect(Dialect):
 
 
 class Compiler(object):
-    def __init__(self, config):
+    ESCAPE_CHARS = ['\\\\a', '\\\\b', '\\\\f', '\\\\n', '\\\\r', '\\\\t', '\\\\v', '\\\\0']
+
+    def __init__(self, config, env_variables):
         self.config = config
+        self.env_variables = env_variables
         self.mapping = {}
 
     def compile(self, sql, arguments):
+        if sql[:4].lower() not in ("set ", "use ") and "\\\\" in sql:
+            for escape_char in self.ESCAPE_CHARS:
+                sql = sql.replace(escape_char, "\\\\\\" + escape_char)
         sql = self.parse_mapping(sql)
         expression = maybe_parse(sql, dialect=CompilerDialect)
         return self.compile_expression(expression, arguments)
@@ -278,7 +285,8 @@ class Compiler(object):
                     continue
                 elif self.is_column(select_expression.args["this"]):
                     column_expression = select_expression.args["this"]
-                elif isinstance(select_expression.args["this"], (sqlglot_expressions.Count, sqlglot_expressions.Sum, sqlglot_expressions.Min, sqlglot_expressions.Max)):
+                elif isinstance(select_expression.args["this"], (sqlglot_expressions.Count, sqlglot_expressions.Sum, sqlglot_expressions.Min,
+                                                                 sqlglot_expressions.Max)):
                     aggregate_expression = select_expression.args["this"]
                 else:
                     calculate_expression = select_expression.args["this"]
@@ -287,12 +295,12 @@ class Compiler(object):
                 column_alias = str(select_expression)
                 config["schema"][column_alias] = self.compile_const(const_info)
                 continue
-            elif isinstance(select_expression, (sqlglot_expressions.Anonymous, sqlglot_expressions.Binary, sqlglot_expressions.If, sqlglot_expressions.Case)):
-                column_alias = str(select_expression)
-                calculate_expression = select_expression
             elif isinstance(select_expression, (sqlglot_expressions.Count, sqlglot_expressions.Sum, sqlglot_expressions.Min, sqlglot_expressions.Max)):
                 column_alias = str(select_expression)
                 aggregate_expression = select_expression
+            elif self.is_calculate(select_expression):
+                column_alias = str(select_expression)
+                calculate_expression = select_expression
             else:
                 raise SyncanySqlCompileException("table select field must be alias: " + self.to_sql(expression))
             if column_expression:
@@ -305,8 +313,14 @@ class Compiler(object):
 
             calculate_fields = []
             self.parse_calculate(primary_table, calculate_expression, calculate_fields)
-            calculate_table_names = {calculate_field["table_name"] for calculate_field in calculate_fields
-                                     if calculate_field["table_name"] and calculate_field["table_name"] != primary_table["table_name"]}
+            calculate_table_names = set([])
+            for calculate_field in calculate_fields:
+                if not calculate_field["table_name"] or calculate_field["table_name"] == primary_table["table_name"]:
+                    continue
+                if calculate_field["table_name"] not in join_tables:
+                    raise SyncanySqlCompileException("table select field join table %s unknown: %s" %
+                                                     (calculate_field["table_name"], self.to_sql(expression)))
+                calculate_table_names.add(calculate_field["table_name"])
             if calculate_table_names:
                 column_join_tables = []
                 self.compile_join_column_tables(primary_table, [join_tables[calculate_table_name] for calculate_table_name in calculate_table_names],
@@ -340,17 +354,9 @@ class Compiler(object):
             self.compile_group_column(primary_table, config, group_expression, group_fields, join_tables)
         config["input"] = "".join(["&.", primary_table["db"], ".", primary_table["name"], "::",
                                    "+".join(primary_table["loader_primary_keys"]) if primary_table["loader_primary_keys"] else "id"])
-        outputer_type = ""
-        if " use " in config["output"]:
-            outputer_type = " use " + config["output"].split(" use ")[-1]
-        elif not isinstance(config["schema"], dict) or [key for key in primary_table["outputer_primary_keys"] if
-                                                      key not in config["schema"]] or not config.get("querys"):
-            outputer_type = " use I"
-        elif [key for key in config["querys"] if key not in config["schema"]]:
-            outputer_type = " use UI"
         config["output"] = "".join([config["output"].split("::")[0], "::",
                                     "+".join(primary_table["outputer_primary_keys"]) if primary_table["outputer_primary_keys"] else "id",
-                                    outputer_type])
+                                    (" use " + config["output"].split(" use ")[-1]) if " use " in config["output"] else " use I"])
 
     def compile_pipleline_select(self, expression, config, arguments, primary_table):
         select_expressions = expression.args.get("expressions")
@@ -372,17 +378,9 @@ class Compiler(object):
         config["pipelines"].append(pipeline)
         config["input"] = "".join(["&.", primary_table["db"], ".", primary_table["name"], "::",
                                    "+".join(primary_table["loader_primary_keys"]) if primary_table["loader_primary_keys"] else "id"])
-        outputer_type = ""
-        if " use " in config["output"]:
-            outputer_type = " use " + config["output"].split(" use ")[-1]
-        elif not isinstance(config["schema"], dict) or [key for key in primary_table["outputer_primary_keys"] if
-                                                      key not in config["schema"]] or not config.get("querys"):
-            outputer_type = " use I"
-        elif [key for key in config["querys"] if key not in config["schema"]]:
-            outputer_type = " use UI"
         config["output"] = "".join([config["output"].split("::")[0], "::",
                                     "+".join(primary_table["outputer_primary_keys"]) if primary_table["outputer_primary_keys"] else "id",
-                                    outputer_type])
+                                    (" use " + config["output"].split(" use ")[-1]) if " use " in config["output"] else " use I"])
         arguments["@primary_order"] = False
         return config
 
@@ -390,7 +388,10 @@ class Compiler(object):
         column_info = self.parse_column(column_expression)
         if not column_alias:
             column_alias = column_info["column_name"].replace(".", "_")
-        if column_info["table_name"] and column_info["table_name"] != primary_table["table_name"] and column_info["table_name"] in join_tables:
+        if column_info["table_name"] and column_info["table_name"] != primary_table["table_name"]:
+            if column_info["table_name"] not in join_tables:
+                raise SyncanySqlCompileException("table select field join table %s unknown: %s" %
+                                                 (column_info["table_name"], self.to_sql(column_expression)))
             column_join_tables = []
             self.compile_join_column_tables(primary_table, [join_tables[column_info["table_name"]]], join_tables,
                                             column_join_tables)
@@ -562,7 +563,12 @@ class Compiler(object):
                             and group_field["table_name"] != primary_table["table_name"]]
         column_join_tables = []
         if calculate_fields:
-            calculate_table_names = {calculate_field["table_name"] for calculate_field in calculate_fields}
+            calculate_table_names = set([])
+            for calculate_field in calculate_fields:
+                if calculate_field["table_name"] not in join_tables:
+                    raise SyncanySqlCompileException("table select field join table %s unknown: %s" %
+                                                     (calculate_field["table_name"], self.to_sql(group_expression)))
+                calculate_table_names.add(calculate_field["table_name"])
             self.compile_join_column_tables(primary_table, [join_tables[calculate_table_name] for calculate_table_name in calculate_table_names],
                                             join_tables, column_join_tables)
         group_column = ["@add", ["#const", "k_"]]
@@ -591,7 +597,12 @@ class Compiler(object):
                             and calculate_field["table_name"] != primary_table["table_name"]]
         column_join_tables = []
         if calculate_fields:
-            calculate_table_names = {calculate_field["table_name"] for calculate_field in calculate_fields}
+            calculate_table_names = set([])
+            for calculate_field in calculate_fields:
+                if calculate_field["table_name"] not in join_tables:
+                    raise SyncanySqlCompileException("table select field join table %s unknown: %s" %
+                                                     (calculate_field["table_name"], self.to_sql(aggregate_expression)))
+                calculate_table_names.add(calculate_field["table_name"])
             self.compile_join_column_tables(primary_table, [join_tables[calculate_table_name] for calculate_table_name in calculate_table_names],
                                             join_tables, column_join_tables)
         group_column = ["@add", ["#const", "k_"]]
@@ -639,7 +650,9 @@ class Compiler(object):
             raise SyncanySqlCompileException("unkonw calculate: " + self.to_sql(expression))
         
     def compile_calculate(self, primary_table, expression, column_join_tables, join_index=-1):
-        if isinstance(expression, sqlglot_expressions.Anonymous):
+        if isinstance(expression, sqlglot_expressions.Neg):
+            return ["@neg", self.compile_calculate(primary_table, expression.args["this"], column_join_tables, join_index)]
+        elif isinstance(expression, sqlglot_expressions.Anonymous):
             calculater_name = expression.args["this"].lower()
             if calculater_name == "get_value":
                 get_value_expressions = expression.args.get("expressions")
@@ -684,13 +697,37 @@ class Compiler(object):
                         column.append(self.compile_calculate(primary_table, arg_expression, column_join_tables, join_index))
                 return column
 
-            column = ["@" + "::".join(calculater_name.split("$"))]
+            if is_mysql_func(calculater_name):
+                column = ["@mysql::" + calculater_name]
+            else:
+                if calculater_name[:8] == "convert_":
+                    column = ["@" + "::".join(calculater_name.split("$")) + "|" + calculater_name[8:]]
+                else:
+                    column = ["@" + "::".join(calculater_name.split("$"))]
             for arg_expression in expression.args.get("expressions", []):
                 if self.is_const(arg_expression):
                     column.append(self.parse_const(arg_expression)["value"])
                 else:
                     column.append(self.compile_calculate(primary_table, arg_expression, column_join_tables, join_index))
             return column
+        elif isinstance(expression, sqlglot_expressions.Cast):
+            to_type = expression.args["to"].args["this"]
+            if to_type in sqlglot_expressions.DataType.FLOAT_TYPES:
+                typing_filter = "float"
+            elif to_type in sqlglot_expressions.DataType.INTEGER_TYPES:
+                typing_filter = "int"
+            elif to_type == sqlglot_expressions.DataType.Type.DATE:
+                typing_filter = "date"
+            elif to_type in sqlglot_expressions.DataType.TEMPORAL_TYPES:
+                typing_filter = "datetime"
+            elif to_type == sqlglot_expressions.DataType.TEXT_TYPES:
+                typing_filter = "str"
+            else:
+                typing_filter = None
+            value_column = self.compile_calculate(primary_table, expression.args["this"], column_join_tables, join_index)
+            if typing_filter:
+                return ["#if", ["#const", True], value_column, None, ":$.*|" + typing_filter]
+            return value_column
         elif isinstance(expression, sqlglot_expressions.Binary):
             return [
                 "@" + expression.key.lower(),
@@ -704,6 +741,24 @@ class Compiler(object):
                 self.compile_calculate(primary_table, expression.args["true"], column_join_tables, join_index) if expression.args.get("true") else False,
                 self.compile_calculate(primary_table, expression.args["false"], column_join_tables, join_index) if expression.args.get("false") else False,
             ]
+        elif isinstance(expression, (sqlglot_expressions.IfNull, sqlglot_expressions.Coalesce)):
+            condition_column = self.compile_calculate(primary_table, expression.args["this"], column_join_tables, join_index)
+            if isinstance(expression, sqlglot_expressions.Coalesce):
+                def parse_coalesce(coalesce_expressions):
+                    if not coalesce_expressions:
+                        return None
+                    coalesce_column = self.compile_calculate(primary_table, coalesce_expressions[0], column_join_tables, join_index)
+                    if len(coalesce_expressions) == 1:
+                        return coalesce_column
+                    if len(coalesce_expressions) > 2:
+                        coalesce_value_column = parse_coalesce(coalesce_expressions[1:])
+                    else:
+                        coalesce_value_column = self.compile_calculate(primary_table, coalesce_expressions[1], column_join_tables, join_index)
+                    return ["#if", ["@is_null", coalesce_column], coalesce_value_column, coalesce_column]
+                value_column = parse_coalesce(expression.args.get("expressions"))
+            else:
+                value_column = self.compile_calculate(primary_table, expression.args["expression"], column_join_tables, join_index)
+            return ["#if", ["@is_null", condition_column], value_column, condition_column]
         elif isinstance(expression, sqlglot_expressions.Case):
             cases = {}
             cases["#case"] = self.compile_calculate(primary_table, expression.args["this"], column_join_tables, join_index) \
@@ -723,11 +778,16 @@ class Compiler(object):
                 "substring": ["this", "start", "length"],
             }
             func_name = expression.key.lower()
-            column, arg_names = ["@" + func_name], (func_args.get(func_name) or ["this", "expression", "expressions"])
+            column = ["@mysql::" + func_name] if is_mysql_func(func_name) else ["@" + func_name]
+            arg_names = func_args.get(func_name) or [key for key in expression.arg_types] or ["this", "expression", "expressions"]
             for arg_name in arg_names:
                 if not expression.args.get(arg_name):
                     continue
-                column.append(self.compile_calculate(primary_table, expression.args[arg_name], column_join_tables, join_index))
+                if isinstance(expression.args[arg_name], list):
+                    for item_expression in expression.args[arg_name]:
+                        column.append(self.compile_calculate(primary_table, item_expression, column_join_tables, join_index))
+                else:
+                    column.append(self.compile_calculate(primary_table, expression.args[arg_name], column_join_tables, join_index))
             return column
         elif isinstance(expression, sqlglot_expressions.Paren):
             return self.compile_calculate(primary_table, expression.args["this"], column_join_tables, join_index)
@@ -739,6 +799,8 @@ class Compiler(object):
         elif isinstance(expression, sqlglot_expressions.Tuple):
             return ["#const", [self.parse_const(tuple_expression)["value"] for tuple_expression in expression.args["expressions"]
                                if self.is_const(tuple_expression)]]
+        elif isinstance(expression, sqlglot_expressions.Interval):
+            return ["#const", {"value": expression.args["this"].args["this"], "unit": expression.args["unit"].args["this"]}]
         elif self.is_const(expression):
             return self.compile_const(self.parse_const(expression))
         else:
@@ -1056,6 +1118,8 @@ class Compiler(object):
         if isinstance(expression, sqlglot_expressions.Anonymous):
             for arg_expression in expression.args.get("expressions", []):
                 self.parse_calculate(primary_table, arg_expression, calculate_fields)
+        elif isinstance(expression, sqlglot_expressions.Cast):
+            self.parse_calculate(primary_table, expression.args["this"], calculate_fields)
         elif isinstance(expression, sqlglot_expressions.If):
             self.parse_calculate(primary_table, expression.args["this"], calculate_fields)
             if expression.args.get("true"):
@@ -1077,7 +1141,7 @@ class Compiler(object):
             if not column["table_name"] or primary_table["table_name"] == column["table_name"]:
                 primary_table["columns"][column["column_name"]] = column
             calculate_fields.append(column)
-        elif isinstance(expression, sqlglot_expressions.Star):
+        elif isinstance(expression, (sqlglot_expressions.Star, sqlglot_expressions.Interval, sqlglot_expressions.DataType)):
             pass
         elif self.is_const(expression):
             pass
@@ -1089,6 +1153,16 @@ class Compiler(object):
             if "expressions" in expression.args and expression.args["expressions"] and isinstance(expression.args["expressions"], list):
                 for arg_expression in expression.args.get("expressions", []):
                     self.parse_calculate(primary_table, arg_expression, calculate_fields)
+            for arg_type in expression.arg_types:
+                if arg_type in ("this", "expression", "expressions"):
+                    continue
+                if arg_type not in expression.args or not expression.args[arg_type]:
+                    continue
+                if isinstance(expression.args[arg_type], list):
+                    for arg_expression in expression.args.get(arg_type, []):
+                        self.parse_calculate(primary_table, arg_expression, calculate_fields)
+                else:
+                    self.parse_calculate(primary_table, expression.args[arg_type], calculate_fields)
 
     def parse_table(self, expression, config):
         db_name = expression.args["db"].name if "db" in expression.args and expression.args["db"] else None
@@ -1217,16 +1291,44 @@ class Compiler(object):
         is_neg, typing_filter = False, None
         if isinstance(expression, sqlglot_expressions.Neg):
             is_neg, expression = True, expression.args["this"]
-        if isinstance(expression, sqlglot_expressions.Literal):
+        if isinstance(expression, (sqlglot_expressions.Literal, sqlglot_expressions.ByteString)):
             value = expression.args["this"]
             if expression.is_number:
                 value = int(value) if expression.is_int else float(value)
                 if is_neg:
                     value = -value
                 typing_filter = "int" if expression.is_int else "float"
+            elif isinstance(value, str):
+                try:
+                    escape_values, escape_chars, encoding = [], [], os.environ.get("SYNCANYENCODING", "utf-8")
+                    for c in value:
+                        if ord(c) <= 256:
+                            escape_chars.append(c)
+                            continue
+                        if escape_chars:
+                            escape_values.append("".join(escape_chars).encode(encoding).decode("unicode_escape"))
+                            escape_chars = []
+                        escape_values.append(c)
+                    if escape_chars:
+                        escape_values.append("".join(escape_chars).encode(encoding).decode("unicode_escape"))
+                    value = "".join(escape_values)
+                except:
+                    pass
         elif isinstance(expression, sqlglot_expressions.Boolean):
             value = expression.args["this"]
             typing_filter = "bool"
+        elif isinstance(expression, (sqlglot_expressions.HexString, sqlglot_expressions.HexString)):
+            value, typing_filter = int(expression.args["this"]), "int"
+        elif isinstance(expression, sqlglot_expressions.Parameter):
+            name = expression.args["this"].args["this"]
+            if name in self.mapping:
+                name = self.mapping[name]
+            try:
+                value = self.env_variables.get_value("@" + name)
+                if isinstance(value, (int, float, bool)):
+                    typing_filter = str(type(value).__name__)
+            except KeyError:
+                raise SyncanySqlCompileException("unkonw parameter: " + self.to_sql(expression))
         else:
             value = None
         return {
@@ -1318,8 +1420,16 @@ class Compiler(object):
         return parse_dot(expression)
 
     def is_const(self, expression):
-        return isinstance(expression, (sqlglot_expressions.Neg, sqlglot_expressions.Literal, sqlglot_expressions.Boolean,
-                                       sqlglot_expressions.Null))
+        if isinstance(expression, sqlglot_expressions.Neg):
+            return not isinstance(expression.args["this"], sqlglot_expressions.Neg) and self.is_const(expression.args["this"])
+        return isinstance(expression, (sqlglot_expressions.Literal, sqlglot_expressions.Boolean,
+                                       sqlglot_expressions.Null, sqlglot_expressions.HexString, sqlglot_expressions.BitString,
+                                       sqlglot_expressions.ByteString, sqlglot_expressions.Parameter))
+
+    def is_calculate(self, expression):
+        return isinstance(expression, (sqlglot_expressions.Neg, sqlglot_expressions.Anonymous, sqlglot_expressions.Binary, sqlglot_expressions.If,
+                                       sqlglot_expressions.IfNull, sqlglot_expressions.Coalesce, sqlglot_expressions.Case,
+                                       sqlglot_expressions.Func, sqlglot_expressions.Cast))
 
     def to_sql(self, expression_sql):
         if not isinstance(expression_sql, str):
