@@ -4,6 +4,7 @@
 
 import os
 import sys
+from collections import defaultdict
 from pygments.lexers import SqlLexer
 from prompt_toolkit import PromptSession
 from prompt_toolkit.history import FileHistory
@@ -12,7 +13,6 @@ from prompt_toolkit.lexers import PygmentsLexer
 from prompt_toolkit.styles import Style
 from prompt_toolkit.filters import Condition
 from prompt_toolkit.cursor_shapes import CursorShape
-from syncanysql.executor import Executor
 from .version import version
 from .parser import SqlParser, SqlSegment
 
@@ -155,20 +155,59 @@ style = Style.from_dict(
     }
 )
 
+
+class CliFileHistory(FileHistory):
+    def load_history_strings(self):
+        strings, time_strings, lines, time_line = [], [], [], ''
+        if os.path.exists(self.filename):
+            with open(self.filename, "rb") as f:
+                for line_bytes in f:
+                    line = line_bytes.decode("utf-8", errors="replace")
+                    if line.startswith("#"):
+                        time_line = line
+                    elif line.startswith("+"):
+                        lines.append(line[1:])
+                        continue
+                    if not lines:
+                        continue
+                    string, lines = "".join(lines)[:-1], []
+                    strings.append(string)
+                    time_strings.append(time_line)
+
+                if lines:
+                    string, lines = "".join(lines)[:-1], []
+                    strings.append(string)
+                    time_strings.append(time_line)
+
+        if len(strings) <= 50:
+            return reversed(strings)
+        with open(self.filename, "wb") as f:
+            for i in range(50):
+                f.write(("\n%s" % time_strings[i - 50]).encode("utf-8"))
+                for line in strings[i - 50].split("\n"):
+                    f.write(("+%s\n" % line).encode("utf-8"))
+        return reversed(strings)
+    
+    def store_string(self, string):
+        if string.lower()[:4] == "exit":
+            return 
+        super(CliFileHistory, self).store_string(string)
+
+
 class CliPrompt(object):
-    def __init__(self, manager, session_config):
+    def __init__(self, manager, session_config, executor):
         self.manager = manager
         self.session_config = session_config
+        self.executor = executor
 
     def run(self):
         home_config_path = os.path.join(os.path.expanduser('~'), ".syncany")
         if not os.path.exists(home_config_path):
             os.mkdir(home_config_path)
-        history = FileHistory(os.path.join(home_config_path, "history"))
+        history = CliFileHistory(os.path.join(home_config_path, "history"))
         session = PromptSession(
             lexer=PygmentsLexer(SqlLexer), completer=sql_completer, style=style, history=history
         )
-        executor = Executor(self.manager, self.session_config)
         print("Python %s" % sys.version)
         print("Syncany-SQL %s -- Simple and easy-to-use sql execution engine" % version)
         lineno = 1
@@ -181,12 +220,12 @@ class CliPrompt(object):
                     continue
                 if text.lower()[:4] == "exit":
                     return 0
-                executor.run("cli", [SqlSegment(text, lineno)])
+                self.executor.run("cli", [SqlSegment(text, lineno)])
                 try:
-                    executor.execute()
+                    self.executor.execute()
                 finally:
                     lineno += 1
-                    executor.runners.clear()
+                    self.executor.runners.clear()
             except KeyboardInterrupt:
                 continue  # Control-C pressed. Try again.
             except EOFError:
